//
// Created by philipp on 29.11.17.
//

#ifndef PROTOCOL_PLATOONINGPROTOCOL_H
#define PROTOCOL_PLATOONINGPROTOCOL_H

#include "Definitions.h"

#include <cstdint>
#include <string>
#include <iostream>
#include <mutex>

namespace platoonProtocol
{
using VehicleId = std::uint32_t;
using MessageType = std::uint32_t;
using PlatoonId = std::uint32_t;
using PlatoonSpeed = float;
using InnerPlatoonDistance = float;

constexpr std::uint16_t UDP_PORT{10000};
constexpr std::uint16_t TCP_PORT{10000};

struct PlatoonConfig
{
    PlatoonConfig() : platoonSpeed(0.0f), innerPlatoonDistance(0.0f)
    {}

    PlatoonConfig(PlatoonSpeed platoonSpeed, InnerPlatoonDistance innerPlatoonDistance)
        : platoonSpeed(platoonSpeed), innerPlatoonDistance(innerPlatoonDistance)
    {}

    PlatoonSpeed platoonSpeed;
    InnerPlatoonDistance innerPlatoonDistance;
};

bool operator==(const PlatoonConfig & lhs, const PlatoonConfig & rhs) noexcept;

bool operator!=(const PlatoonConfig & lhs, const PlatoonConfig & rhs) noexcept;

template<typename... Ts>
inline void log(const Ts & ... args)
{
#ifdef PLATOON_PROTOCOL_ENABLE_LOGGING
    static std::mutex logMutex;
    std::lock_guard<std::mutex> lock(logMutex);
    auto l = {(PLATOON_PROTOCOL_LOGGING_STREAM << args, true)...};
    std::cout.flush();
#endif
};

}

#endif //PROTOCOL_PLATOONINGPROTOCOL_H
